/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2004 Brice Goglin <Brice.Goglin@ens-lyon.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFS_MOUNT_H__
#define __ORFS_MOUNT_H__


#ifdef __KERNEL__
#include <linux/in.h>
#else
#include <arpa/inet.h>
#include <asm/param.h>
#endif

#define ORFS_MAX_SERVER_TYPE_LEN 7

#define ORFS_SERVER_TYPE_BIT 0
#define ORFS_SERVER_PORT_BIT 1
#define ORFS_SERVER_EP_BIT 2
#define ORFS_SERVER_BOARD_BIT 3
#define ORFS_CLIENT_BOARD_BIT 4
#define ORFS_SERVER_TCP_SOCKADDR_BIT 5

#define ORFS_MULTICLIENT_BIT 10
#define ORFS_ALWAYS_O_DIRECT_BIT 11

#define ORFS_DENTRY_REVALIDATE_DELAY_BIT 20
#define ORFS_INODE_REVALIDATE_DELAY_BIT 21
#define ORFS_PAGE_REVALIDATE_DELAY_BIT 22
#define ORFS_NON_CONCURRENT_BIT 23

#define ORFS_SERVER_TYPE_MASK (1UL << ORFS_SERVER_TYPE_BIT)
#define ORFS_SERVER_PORT_MASK (1UL << ORFS_SERVER_PORT_BIT)
#define ORFS_SERVER_EP_MASK (1UL << ORFS_SERVER_EP_BIT)
#define ORFS_SERVER_BOARD_MASK (1UL << ORFS_SERVER_BOARD_BIT)
#define ORFS_CLIENT_BOARD_MASK (1UL << ORFS_CLIENT_BOARD_BIT)
#define ORFS_SERVER_TCP_SOCKADDR_MASK (1UL << ORFS_SERVER_TCP_SOCKADDR_BIT)
#define ORFS_MULTICLIENT_MASK (1UL << ORFS_MULTICLIENT_BIT)
#define ORFS_ALWAYS_O_DIRECT_MASK (1UL << ORFS_ALWAYS_O_DIRECT_BIT)
#define ORFS_DENTRY_REVALIDATE_DELAY_MASK (1UL << ORFS_DENTRY_REVALIDATE_DELAY_BIT)
#define ORFS_INODE_REVALIDATE_DELAY_MASK (1UL << ORFS_INODE_REVALIDATE_DELAY_BIT)
#define ORFS_PAGE_REVALIDATE_DELAY_MASK (1UL << ORFS_PAGE_REVALIDATE_DELAY_BIT)
#define ORFS_NON_CONCURRENT_MASK (1UL << ORFS_NON_CONCURRENT_BIT)

#define ORFS_REMOUNT_SPECIFIC_MASK 		\
	(ORFS_ALWAYS_O_DIRECT_MASK		\
	|ORFS_DENTRY_REVALIDATE_DELAY_MASK	\
	|ORFS_INODE_REVALIDATE_DELAY_MASK	\
	|ORFS_PAGE_REVALIDATE_DELAY_MASK	\
	|ORFS_NON_CONCURRENT_MASK		\
	)

struct orfs_mount_flagmask {
  unsigned long flags;
  unsigned long mask;
};

struct orfs_mount_revalidate_delay {
  unsigned long dentry;
  unsigned long inode;
  unsigned long page;
};

struct orfs_common_mount_connection {
  char layer[ORFS_MAX_SERVER_TYPE_LEN+1];
  char name[MAXHOSTNAMELEN];      
};

struct orfs_tcp_mount_connection {
  unsigned long port;
  struct sockaddr_in sockaddr_in;
};

struct orfs_gm_mount_connection {
  unsigned long port;
  unsigned long board;
  unsigned long client_board;
};

struct orfs_mx_mount_connection {
  unsigned long ep;
  unsigned long board;
  unsigned long client_board;
};

struct orfs_user_mount_options {
  /* which field have been set */
  struct orfs_mount_flagmask specific;
  /* revalidate delays */
  struct orfs_mount_revalidate_delay revalidate_delay;
  /* connection parameters */
  struct orfs_common_mount_connection common;
  struct orfs_tcp_mount_connection tcp;
  struct orfs_gm_mount_connection gm;
  struct orfs_mx_mount_connection mx;
};

struct orfs_user_remount_options {
  /* which field have been set */
  struct orfs_mount_flagmask generic;
  struct orfs_mount_flagmask specific;
  /* revalidate delays */
  struct orfs_mount_revalidate_delay revalidate_delay;
};


#endif /* __ORFS_MOUNT_H__ */
