/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2004 Brice Goglin <Brice.Goglin@ens-lyon.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <errno.h>

#include "orfs_protocol.h"
#include "orfs_common_network.h"
#include "interactive_messages.h"


unsigned long tcp_server_port = ORFS_TCP_SERVER_PORT;

static int main_socket = -1;


int TCP_init(char *name)
{
  struct hostent *hostent;
  struct sockaddr_in address = { 0 };

  if ( (main_socket = socket(PF_INET, SOCK_STREAM, 0)) < 0 ) {
    ERROR("Socket creation failed with error %m\n");
    return -1;
  }
  
  if ( (hostent = gethostbyname(name)) == NULL ) {
    ERROR("Cannot find server (%m)\n");
    return -1;
  }
  
  address.sin_family = hostent->h_addrtype;
  address.sin_port = htons(tcp_server_port);
  memcpy(&address.sin_addr,
	 hostent->h_addr,
	 hostent->h_length);
  
  if ( connect(main_socket,
	       (struct sockaddr*)&address,
	       sizeof(address)) < 0 ) {
    ERROR("Connection failed on error %m\n");
    return -1;
  }
  
  return 0;
}


int TCP_request(struct orfs_request *req,
		const void *in_buf,
		struct orfs_answer *ans,
		void *out_buf,
		size_t out_buf_len)
{
  int res;
  void *ptr;
  size_t to_send, to_recv;

  /* send the request */
  res = write(main_socket, req, sizeof(*req));
  if ( res <= 0 )
    goto out;

  /* send request data */
  if ( get_request_buffer_len(req, &to_send) ) {
    res = write(main_socket, in_buf, to_send);
    if ( res <= 0 )
      goto out;
  }

  /* recv the answer */
  ptr = ans;
  to_recv = sizeof(*ans);
  while ( to_recv ) {
    res = read(main_socket, ptr, to_recv);
    if ( res <= 0 )
      goto out;
    to_recv -= res;
    ptr += res;
  }

  /* recv answer data */
  if ( get_answer_buffer_len(ans, &to_recv) ) {
    ptr = out_buf;
    while ( to_recv ) {
      res = read(main_socket, ptr, to_recv);
      if ( res <= 0 )
        goto out;
      to_recv -= res;
      ptr += res;
    }
  }
  
  return 0;

 out:
  ERROR("Connection closed by the server ?");
  exit(-1);
}


void TCP_exit()
{
  close(main_socket);
}
