builtin(include,../../common_aclocal.m4)

####################
# KERNEL

# define KERNEL_PATCHLEVEL
AC_DEFUN(
	[AC_KERNEL_PATCHLEVEL],
	[
		AC_REQUIRE([AC_CHECK_KERNEL_SRC]) dnl
		KERNEL_PATCHLEVEL=`echo $KERNEL_RELEASE | cut -d. -f1``echo $KERNEL_RELEASE | cut -d. -f2`
		AC_SUBST([KERNEL_PATCHLEVEL])
	])

# define KERNEL_SUBLEVEL
AC_DEFUN(
	[AC_KERNEL_SUBLEVEL],
	[
		AC_REQUIRE([AC_CHECK_KERNEL_SRC]) dnl
		[KERNEL_SUBLEVEL=`echo $KERNEL_RELEASE | cut -d. -f3 | sed 's/\([0-9]\+\).*/\1/'`]
		AC_SUBST([KERNEL_SUBLEVEL])
	])

# define KBUILD_TYPE
AC_DEFUN(
	[AC_KBUILD_TYPE],
	[
		AC_REQUIRE([AC_KERNEL_PATCHLEVEL]) dnl
		AC_REQUIRE([AC_KERNEL_SUBLEVEL]) dnl
		if test "$KERNEL_PATCHLEVEL" == 24; then
			KBUILD_TYPE="24"
		elif test "$KERNEL_PATCHLEVEL" == 26; then
			if test $KERNEL_SUBLEVEL -le 5; then
				KBUILD_TYPE="26BEFORE5"
			else
				KBUILD_TYPE="26AFTER6"
			fi
		fi
		AC_SUBST([KBUILD_TYPE])	
	])

# check KERNEL is 2.4.x or 2.6.x
AC_DEFUN(
	[AC_CHECK_KERNEL_VERSION],
	[
		AC_REQUIRE([AC_PROG_CPP]) dnl
		AC_REQUIRE([AC_CHECK_KERNEL_SRC]) dnl
		AC_MSG_CHECKING([linux kernel 2.4/2.6])
		AC_TRY_CPP(
			[
#include <$KERNEL_SRC/include/linux/version.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) && LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) && LINUX_VERSION_CODE < KERNEL_VERSION(2,7,0)
#else
#error
#endif
			],
			[AC_MSG_RESULT([yes ($KERNEL_RELEASE)])],
			[
				AC_MSG_RESULT([no ($KERNEL_RELEASE)])
				AC_MSG_ERROR([Kernel version not supported])
			])
	])

# check KERNEL is at least $1.$2.$3 to accept $4 for $5 support
AC_DEFUN(
	[AC_CHECK_KERNEL_VERSION_AT_LEAST],
	[
		AC_REQUIRE([AC_PROG_CPP]) dnl
		AC_REQUIRE([AC_CHECK_KERNEL_SRC]) dnl
		AC_MSG_CHECKING([for $4 support (for $5) in linux kernel ($1.$2.$3 required)])
		AC_TRY_CPP(
			[
#include <$KERNEL_SRC/include/linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION($1,$2,$3)
#error
#endif
			],
			[AC_MSG_RESULT([yes])],
			[
				AC_MSG_RESULT([no])
				AC_MSG_ERROR([Kernel version too old for $5])
			])
	])

# check that kernel supports seq_file interface
# $1	feature that requires this
AC_DEFUN(
	[AC_CHECK_KERNEL_SEQ_FILE_WITH_SINGLE_OPS],
	[AC_CHECK_KERNEL_VERSION_AT_LEAST(2, 4, 23, [seq_file with single ops], [$1])]
	)

####################
# GMKRC

# check GMKRC API
AC_DEFUN(
	[AC_CHECK_GMKRC_SRC],
	[
		AC_MSG_CHECKING([GMKRC source in $GMKRC_SRC])
		GMKRC_VERSION=`grep GMKRC_VERSION ${GMKRC_SRC}/include/gmkrc.h 2> /dev/null | cut -d\" -f2 || true`
		if test "$GMKRC_VERSION" == ""; then
		        AC_MSG_RESULT([no])
			AC_MSG_ERROR([No include/gmkrc.h in $GMKRC_SRC])
		fi
		AC_MSG_RESULT([yes ($GMKRC_VERSION)])
	])

# define CONFIG_ORFS_GMKRC
AC_DEFUN(
	[AC_ENABLE_GMKRC],
	[
		AC_ARG_ENABLE(
			[gmkrc],
			AC_HELP_STRING(
				[--enable-gmkrc],
				[GMKRC layer (default is no)]),
			[AC_MSG_RESULT([activating GMKRC layer... $enableval])],
			[enableval=no])
		if test "$enable_gm" != no -a "$enableval" != no; then
			AC_WITH_KPROG_SRC([gmkrc], "/usr/src/gmkrc")
			AC_CHECK_GMKRC_SRC
			AC_SUBST([GMKRC_SRC])
			AC_DEFINE(
				[CONFIG_ORFS_GMKRC],
				1,
				[Define to 1 to enable GMKRC layer])
			enable_gmkrc=yes
		fi
		AC_SUBST([CONFIG_ORFS_GMKRC], [$enableval])
	])

####################
# GM

# check version
AC_DEFUN(
	[AC_CHECK_GM_SRC],
	[
		AC_MSG_CHECKING([GM source in $GM_SRC])
		GM_VERSION=`grep _gm_version ${GM_SRC}/libgm/_gm_version.c 2> /dev/null | cut -d\" -f2 || true`
		if test "$GM_VERSION" == ""; then
		        AC_MSG_RESULT([no])
			AC_MSG_ERROR([No libgm/_gm_version.c in in $GM_SRC])
		fi
		AC_MSG_RESULT([yes ($GM_VERSION)])
	])

AC_DEFUN(
	[AC_WITH_GM_SRC],
	[
		AC_WITH_KPROG_SRC([gm], "/usr/src/gm")
		AC_CHECK_GM_SRC
	])

##########################
# MX

AC_DEFUN(
	[AC_WITH_MX_HEADERS],
	[
		AC_WITH_PROG([mx])
		AC_WITH_PROG_HDR([mx])
		AC_WITH_PROG_SRC(
			[mx],
			[libmyriexpress],
			[common])
		AC_SUBST([MX_HEADERS])
		AC_CHECK_HEADER([myriexpress.h], [], [AC_MSG_ERROR([myriexpress.h not found])])
	])

##########################
# KERNEL NETWORK

AC_DEFUN(
	[AC_ENABLE_NETWORK_SUBMODULES],
	[
		AC_ARG_ENABLE(
			[netmodules],
			AC_HELP_STRING([--enable-netmodules], [(default is no)]),
			[
				enable_netmodules=$enableval
				AC_MSG_RESULT([activating network layers as submodules... $enableval])
			],
			[enable_netmodules=no])
		if test "$enable_netmodules" == "yes"; then
			AC_DEFINE([CONFIG_ORFS_NETMODULES], 1, [Define to 1 to compile network layers as submodules])
		fi
		AC_SUBST([CONFIG_ORFS_NETMODULES], [$enable_netmodules])
	])

# define LAYERS_SUBMODULES and BUILTIN_LAYERS_SRC in Makefile
# and *_LAYER_SUBMODULE or *_LAYER_BUILTIN in header
# arguments:	$1 = layer
#		$2 = default
#		$3 = required check
AC_DEFUN(
	[AC_WITH_KERNEL_NETLAYER],
	[
		AC_ARG_ENABLE(
			[$1],
			AC_HELP_STRING([--enable-$1], translit([$1], [a-z], [A-Z])[ network layer (default is $2)]),
	       		[AC_MSG_RESULT([activating ]translit([$1], [a-z], [A-Z])[ network layer... $enableval])],
			[enableval=[$2]])

		if test "$enableval" != no; then
			AC_DEFINE(
				[CONFIG_ORFS_]translit([$1], [a-z], [A-Z]),
				1,
				[Define to 1 if ]translit([$1], [a-z], [A-Z])[ network layer enabled])
			$3
		fi
		AC_SUBST([CONFIG_ORFS_]translit([$1], [a-z], [A-Z]), [$enableval])
	])				

# full LAYERS_SUBMODULES and BUILTIN_LAYERS_SRC in Makefile
# and *_LAYER_SUBMODULE or *_LAYER_BUILTIN in header
AC_DEFUN(
	[AC_WITH_KERNEL_NETLAYERS],
	[
		AC_ENABLE_GMKRC
		AC_ENABLE_NETWORK_SUBMODULES
		AC_WITH_KERNEL_NETLAYER([tcp], [yes], [])
		AC_WITH_KERNEL_NETLAYER([gm], [yes], [AC_WITH_GM_SRC])
		AC_WITH_KERNEL_NETLAYER([mx], [yes], [AC_WITH_MX_HEADERS])
	])

###############################
# STANDARD CHECK

# standard AC_ARG_ENABLE with AC_DEFINE
# arguments: 	$1 = underscored named
#		$2 = default value
#		$3 = configure message
#		$4 = help
#		$5 = required check
AC_DEFUN(
	[AC_STD_ENABLE_DEFINE],
	[
		AC_ARG_ENABLE(
			[translit([$1], [_], [-])],
			AC_HELP_STRING([--enable-]translit([$1], [_], [-]), [$4 (default is $2)]),
		       	[
				enable_[$1]=$enableval
				AC_MSG_RESULT([activating [$3]... $enableval])
			],
			[enable_[$1]=[$2]])
		if test "$enable_[$1]" = yes; then
		        AC_DEFINE([CONFIG_ORFS_]translit([$1], [a-z], [A-Z]), 1, [Define to 1 if $3 enabled])
			ifelse($5,,,$5($3))
		fi
		AC_SUBST([CONFIG_ORFS_]translit([$1], [a-z], [A-Z]), $enable_[$1])
	])

##################################
# DEBUG

# define CONFIG_ORFS_DEBUG and _LEVEL
AC_DEFUN(
	[AC_CONFIG_DEBUG],
	[
		AC_DEBUG
		if test "$enable_debug" != no; then
			AC_DEFINE(
				CONFIG_ORFS_DEBUG,
				1,
				[Define to activate debug printing])
			if test "$enable_debug" = yes; then
				debug_level=0
			else
				debug_level=$enable_debug
			fi
			AC_DEFINE_UNQUOTED(
				CONFIG_ORFS_DEBUG_LEVEL,
				$debug_level,
				[Define to the initial value of debug printing level])
		fi
	])
