/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2004 Brice Goglin <Brice.Goglin@ens-lyon.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFS_MESSAGES_H__
#define __ORFS_MESSAGES_H__

/***********************************
 * General Purpose Kernel Messages *
 ***********************************/

#define ORFS_NOTICE(args...) \
printk(KERN_NOTICE "ORFS: " args)

#define ORFS_ERROR(args...) \
printk(KERN_ERR "ORFS Error: " args)

#define ORFS_ALERT(args...) \
printk(KERN_ALERT "ORFS ALERT: " args)

#define ORFS_NOTICE_ONCE(tag, args...)			\
do {							\
static int orfs_notice_once_##tag##_counter = 0;	\
if ( !orfs_notice_once_##tag##_counter ) {		\
ORFS_NOTICE(args);					\
orfs_notice_once_##tag##_counter++;			\
};							\
} while (0)

/**********************
 * Debugging Messages *
 **********************/

#if CONFIG_ORFS_DEBUG

extern unsigned long orfs_debug;

#define ORFS_DEBUG_REQ    0x00000001	/* Main requests */
#define ORFS_DEBUG_FS     0x00000002	/* Internal FS requests */
#define ORFS_DEBUG_CREQ   0x00000004	/* Client requests managing */
#define ORFS_DEBUG_ORFSD  0x00000008	/* ORFSD process */
#define ORFS_DEBUG_PD     0x00000010	/* Private Data */
#define ORFS_DEBUG_PROC   0x00000020	/* Special /proc functionnalities */
#define ORFS_DEBUG_NET    0x00000040	/* Generic Network */
#define ORFS_DEBUG_TCP    0x00000080	/* TCP */
#define ORFS_DEBUG_GM     0x00000100	/* GM */
#define ORFS_DEBUG_MX     0x00000200	/* MX */
#define ORFS_DEBUG_ALL    0xffffffff

#define ORFS_DEBUG(mask, args...)      \
do {                                   \
if ( ORFS_DEBUG_##mask & orfs_debug )  \
printk(KERN_INFO "ORFS Debug: " args); \
} while ( 0 )

#else				/* CONFIG_ORFS_DEBUG */

#define ORFS_DEBUG(args...) do { } while ( 0 )

#endif				/* CONFIG_ORFS_DEBUG */

#endif				/* __ORFS_MESSAGES_H__ */
