/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2004 Brice Goglin <Brice.Goglin@ens-lyon.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFS_MISC_H__
#define __ORFS_MISC_H__

#include <linux/signal.h>
#include <linux/sched.h>
#include <asm/uaccess.h>
#include <asm/processor.h>
#include <linux/spinlock.h>

#include "orfs_portable.h"

/**************************
 * Switching to KERNEL_DS *
 **************************/

#define ORFS_DECLARE_KERNEL_DS() mm_segment_t oldfs

#define ORFS_START_KERNEL_DS()	\
do {				\
	oldfs = get_fs();	\
	set_fs(get_ds());	\
} while ( 0 )

#define ORFS_STOP_KERNEL_DS() set_fs(oldfs)

/********************
 * Blocking signals *
 ********************/

#define ORFS_DECLARE_SIGMASK() sigset_t new_set, saved_set

#define ORFS_INIT_SIGMASK() sigfillset(&new_set)

#define ORFS_START_SIGMASK()					\
do {								\
	unsigned long flags;					\
	spin_lock_irqsave(&ORFS_SIGLOCK(current), flags);	\
	saved_set = current->blocked;				\
	current->blocked = new_set;				\
	ORFS_RECALC_SIGPENDING_CURRENT();			\
	spin_unlock_irqrestore(&ORFS_SIGLOCK(current), flags);	\
} while ( 0 )

#define ORFS_STOP_SIGMASK()					\
do {								\
	unsigned long flags;					\
	spin_lock_irqsave(&ORFS_SIGLOCK(current), flags);	\
	current->blocked = saved_set;				\
	ORFS_RECALC_SIGPENDING_CURRENT();			\
	spin_unlock_irqrestore(&ORFS_SIGLOCK(current), flags);	\
} while ( 0 )

#endif				/* __ORFS_MISC_H__ */
