/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2004 Brice Goglin <Brice.Goglin@ens-lyon.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFS_PORTABLE_H__
#define __ORFS_PORTABLE_H__

#if ORFS_LINUX_24

#define ORFS2KERNEL_TIME(ot,kt) do { kt = (time_t)ot; } while (0)
#define KERNEL2ORFS_TIME(kt,ot) do { ot = (orfs_time_t)kt; } while (0)

typedef kdev_t orfs_portable_kdev_t;

#define ORFS_SIGLOCK(p) ((p)->sigmask_lock)
#define ORFS_RECALC_SIGPENDING_CURRENT() recalc_sigpending(current)

#elif ORFS_LINUX_26

#define ORFS2KERNEL_TIME(ot,kt) do { (kt).tv_sec = (time_t)ot; (kt).tv_nsec = 0; } while (0)
#define KERNEL2ORFS_TIME(kt,ot) do { ot = (orfs_time_t)((kt).tv_sec); } while (0)

typedef dev_t orfs_portable_kdev_t;

#define ORFS_SIGLOCK(p) ((p)->sighand->siglock)
#define ORFS_RECALC_SIGPENDING_CURRENT() recalc_sigpending()

#endif				/* ORFS_LINUX_?? */

#define ORFS2KERNEL_DEV(od) (orfs_portable_kdev_t)(od)
#define KERNEL2ORFS_DEV(kd) (orfs_dev_t)(kd)


#endif				/* __ORFS_PORTABLE_H__ */
