/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2004 Brice Goglin <Brice.Goglin@ens-lyon.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFS_STATS__
#define __ORFS_STATS__

struct orfs_sbi_stat {
	struct {
		unsigned long file;
		unsigned long dentry;
		unsigned long inode;
	} alloc;

	struct {
		struct {
			unsigned long call;
			unsigned long remote;
			unsigned long failed;
		} inode, dentry;
	} reval;

	struct {
		unsigned long setattr;
		unsigned long getattr;
		unsigned long readlink;
		unsigned long open;
		unsigned long close;
		unsigned long o_pread;
		unsigned long o_pwrite;
#if ORFS_LINUX_26
		unsigned long o_aioread;
		unsigned long o_aiowrite;
		unsigned long aiofsync;
#endif
		unsigned long readpage;
		unsigned long writepage;
		unsigned long prepare_write;
		unsigned long commit_write;
		unsigned long preaddir;
		unsigned long fsync;
		unsigned long lookup;
		unsigned long mkreg;
		unsigned long mkdir;
		unsigned long mknod;
		unsigned long unlink;
		unsigned long rmdir;
		unsigned long revalidate;
		unsigned long move;
		unsigned long link;
		unsigned long symlink;
		unsigned long statfs;
	} req;
};

#ifdef CONFIG_ORFS_STATS

#define ORFS_INC_ALLOC_STAT(sbi, x)					\
do {									\
	unsigned long val __attribute__((unused));			\
	val = ++(sbi)->stat.alloc.x;					\
	ORFS_DEBUG(PD, "Allocated " #x " private data #%ld\n", val);	\
} while ( 0 )

#define ORFS_DEC_ALLOC_STAT(sbi, x)		       			\
do {									\
	unsigned long val __attribute__((unused));			\
	val = (sbi)->stat.alloc.x--;					\
	ORFS_DEBUG(PD, "Freed " #x " private data #%ld\n", val);       	\
} while ( 0 )

#define ORFS_GET_ALLOC_STAT(sbi, x) (sbi)->stat.alloc.x

#define ORFS_UPDATE_REVAL_STAT(sbi, x, y) (sbi)->stat.reval.x.y++
#define ORFS_GET_REVAL_STAT(sbi, x, y) (sbi)->stat.reval.x.y

#define ORFS_UPDATE_REQ_STAT(sbi, x) (sbi)->stat.req.x++
#define ORFS_GET_REQ_STAT(sbi, x) (sbi)->stat.req.x

struct orfs_sb_info;

extern void orfs_sbi_reset_stats(struct orfs_sb_info *sbi);

#else				/* ~CONFIG_ORFS_STATS */

#define ORFS_INC_ALLOC_STAT(sbi, x)	/* nothing */
#define ORFS_DEC_ALLOC_STAT(sbi, x)	/* nothing */
#define ORFS_GET_ALLOC_STAT(sbi, x) 0UL
#define ORFS_UPDATE_REVAL_STAT(sbi, x, y)	/* nothing */
#define ORFS_GET_REVAL_STAT(sbi, x, y) 0UL
#define ORFS_UPDATE_REQ_STAT(sbi, x)	/* nothing */
#define ORFS_GET_REQ_STAT(sbi, x) 0UL

#endif				/* ~CONFIG_ORFS_STATS */


#endif				/* __ORFS_STATS__ */
