/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2004 Brice Goglin <Brice.Goglin@ens-lyon.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <linux/version.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>

#include "orfs_fs.h"
#include "orfs_network.h"
#include "orfs_request.h"
#include "orfs_messages.h"

#ifdef CONFIG_ORFS_DEBUG

/* debug mask */
unsigned long orfs_debug = CONFIG_ORFS_DEBUG_LEVEL;

#if ORFS_LINUX_24
MODULE_PARM(orfs_debug, "l");
#elif ORFS_LINUX_26
#include <linux/moduleparam.h>
module_param(orfs_debug, ulong, 0644);
#endif

#endif				/* CONFIG_ORFS_DEBUG */

static int
orfs_init(void)
{
	int err;

	err = orfs_init_fs();
	if (err) {
		return err;
	}

	ORFS_NOTICE("initialized\n");

	return 0;
}

static void
orfs_exit(void)
{
	orfs_exit_fs();

	ORFS_NOTICE("cleaned up\n");
}

#ifdef CONFIG_ORFS_NETMODULES
EXPORT_SYMBOL(orfs_register_net_layer);
EXPORT_SYMBOL(orfs_unregister_net_layer);
EXPORT_SYMBOL(orfs_killall_client_requests);
EXPORT_SYMBOL(orfs_get_client_request);
EXPORT_SYMBOL(orfs_put_client_request);
EXPORT_SYMBOL(orfs_get_client_answer);
EXPORT_SYMBOL(orfs_put_client_answer);
EXPORT_SYMBOL(orfs_launch_orfsd);
EXPORT_SYMBOL(orfs_finish_orfsd);
#ifdef CONFIG_ORFS_DEBUG
EXPORT_SYMBOL(orfs_debug);
#endif
#endif				/* CONFIG_ORFS_NETMODULES */

MODULE_AUTHOR("Brice Goglin <Brice.Goglin@ens-lyon.fr>");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("ORFA Client");

module_init(orfs_init);
module_exit(orfs_exit);
