####################
# KERNEL

# define KERNEL_SRC
AC_DEFUN(
	[AC_WITH_KERNEL_SRC],
	[
		AC_ARG_WITH(
			[linux],
			AC_HELP_STRING([--with-linux=<dir>],[ (default is /usr/src/linux)]),
			[KERNEL_SRC="$withval"],
			[KERNEL_SRC="/usr/src/linux"])
		AC_SUBST([KERNEL_SRC])
	])

# check UTS_RELEASE in ${KERNEL_SRC}/include/linux/version.h
AC_DEFUN(
	[AC_CHECK_KERNEL_SRC],
	[
		AC_REQUIRE([AC_WITH_KERNEL_SRC]) dnl
		AC_MSG_CHECKING([linux kernel source in $KERNEL_SRC])
		KERNEL_RELEASE=`grep UTS_RELEASE ${KERNEL_SRC}/include/linux/version.h 2> /dev/null | cut -d\" -f2 || true`
		if test "$KERNEL_RELEASE" == ""; then
		        AC_MSG_RESULT([no])
			AC_MSG_ERROR([No include/linux/version.h in $KERNEL_SRC])
		fi
		AC_MSG_RESULT([yes])
		AC_SUBST([KERNEL_RELEASE])
	])

########################################
# USER LEVEL LIB

# define $1 library and header path with --with-$1=
AC_DEFUN(
	[AC_WITH_PROG],
	[
		AC_ARG_WITH(
			[[$1]],
			AC_HELP_STRING([--with-[$1]=<dir>],[]),
			[
				LIBS="$LIBS -L $withval/lib"
				CPPFLAGS="$CPPFLAGS -I $withval/include"
				translit([$1], [a-z], [A-Z])[_HEADERS="$withval/include"]
				AC_MSG_RESULT([using $1 installed in... $withval])
			])
	])

# define $1 library path with --with-$1-lib=
AC_DEFUN(
	[AC_WITH_PROG_LIB],
	[
		AC_ARG_WITH(
			[[$1]-lib],
			AC_HELP_STRING([--with-[$1]-lib=<dir>],[]),
			[
				LIBS="$LIBS -L $withval"
				AC_MSG_RESULT([using $1 lib in... $withval])
			])
	])

# define $1 header path with --with-$1-hdr=
AC_DEFUN(
	[AC_WITH_PROG_HDR],
	[
		AC_ARG_WITH(
			[[$1]-hdr],
			AC_HELP_STRING([--with-[$1]-hdr=<dir>],[]),
			[
				CPPFLAGS="$CPPFLAGS -I $withval"
				translit([$1], [a-z], [A-Z])[_HEADERS="$withval"]
				AC_MSG_RESULT([using $1 headers in... $withval])
			])
	])

# define $1 library and header path with --with-$1-src=
# $2 is relative path from source to lib
# $3 is relative path from source to headers
AC_DEFUN(
	[AC_WITH_PROG_SRC],
	[
		AC_ARG_WITH(
			[[$1]-src],
			AC_HELP_STRING([--with-[$1]-src=<dir>],[]),
			[
				LIBS="$LIBS -L $withval/$2"
				CPPFLAGS="$CPPFLAGS -I $withval/$3"
				translit([$1], [a-z], [A-Z])[_HEADERS="$withval/$3"]
				AC_MSG_RESULT([using $1 source in... $withval])
			])
	])

# define kernel program $1 src path with --with-$1-src=
# $2 is default
AC_DEFUN(
	[AC_WITH_KPROG_SRC],
	[
		AC_ARG_WITH(
			[[$1]-src],
			AC_HELP_STRING([--with-[$1]-src=<dir>],translit([$1], [a-z], [A-Z])[ network layer source (default is $2)]),
			translit([$1], [a-z], [A-Z])[_SRC="$withval"],
			translit([$1], [a-z], [A-Z])[_SRC="$2"])
		AC_SUBST(translit([$1], [a-z], [A-Z])[_SRC])
	])

# define and check GM library and headers
AC_DEFUN(
	[AC_WITH_GM],
	[
		AC_WITH_PROG([gm])
		AC_WITH_PROG_LIB([gm])
		AC_WITH_PROG_HDR([gm])
		AC_WITH_PROG_SRC(
			[gm],
			[binary/.gm_uninstalled_libs/lib/.libs],
			[binary/.gm_uninstalled_other/include])
		AC_CHECK_LIB([gm], [gm_init], [], [AC_MSG_ERROR([libgm not found])])
		AC_CHECK_HEADER([gm.h], [], [AC_MSG_ERROR([gm.h not found])])
	])

# define and check MX library and headers
AC_DEFUN(
	[AC_WITH_MX],
	[
		AC_WITH_PROG([mx])
		AC_WITH_PROG_LIB([mx])
		AC_WITH_PROG_HDR([mx])
		AC_WITH_PROG_SRC(
			[mx],
			[libmyriexpress/userspace],
			[common])
		AC_SUBST([MX_HEADERS])
		AC_CHECK_LIB([myriexpress], [mx_init], [], [AC_MSG_ERROR([libmyriexpress not found])], [-lpthread])
		AC_CHECK_HEADER([myriexpress.h], [], [AC_MSG_ERROR([myriexpress.h not found])])
	])

#################################
# USER LEVEL NETWORK

# user-level network layer built-in
# $1	network layer
# $2	default
# $3	required check
AC_DEFUN(
	[AC_ENABLE_NETLAYER],
	[
		AC_ARG_ENABLE(
			[$1],
			[AC_HELP_STRING([--enable-$1], translit([$1], [a-z], [A-Z])[ network layer (default is $2)])],
			[AC_MSG_RESULT([activating ]translit([$1], [a-z], [A-Z])[ network layer... $enableval])],
			[enableval=$2])
		if test "$enableval" != no; then
			$3
			AC_DEFINE(
				[CONFIG_]translit([$1], [a-z], [A-Z]),
				1,
				[Define to 1 if ]translit([$1], [a-z], [A-Z])[ network built-in])
			enableval=yes
		fi
		AC_SUBST([CONFIG_]translit([$1], [a-z], [A-Z]), [$enableval])
	])

# full NETWORK_SRC support in Makefile
# and CONFIG_* in header
AC_DEFUN(
	[AC_WITH_NETWORKS],
	[
		AC_ENABLE_NETLAYER([tcp], [yes], [])
		AC_ENABLE_NETLAYER([gm], [yes], [AC_WITH_GM])
		AC_ENABLE_NETLAYER([mx], [yes], [AC_WITH_MX])
	])

########################################
# DEBUG

# define enable_debug
AC_DEFUN(
	[AC_DEBUG],
	[AC_ARG_ENABLE(
		[debug],
	        AC_HELP_STRING([--enable-debug], [(default is no)]),
        	[
			enable_debug="$enableval"
			AC_MSG_RESULT([activating debug... $enableval])
		],
	        [enable_debug=no])
	])

# define DEBUG_LEVEL and add -g to CFLAGS if needed
AC_DEFUN(
	[AC_DEBUG_LEVEL],
	[
		debug_level=-1
		AC_DEBUG
		if test "$enable_debug" != no; then
			if test "$enable_debug" != yes; then
				debug_level=$enable_debug
			else
				debug_level=10
			fi
		fi
		AC_DEFINE_UNQUOTED(
			DEBUG_LEVEL,
			$debug_level,
			[Define to the quantities of debug printing you want])
		if test "$debug_level" -ge 0; then
			CFLAGS="$CFLAGS -g"
		fi
	])
