/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2004 Brice Goglin <Brice.Goglin@ens-lyon.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __ORFS_TYPES_H__
#define __ORFS_TYPES_H__


#ifndef __KERNEL__
#include <stdint.h>
#endif /* __KERNEL__ */


/****************************
 * Some constants and types *
 ****************************/

/* various contants */
#define ORFS_NAME_LEN 255 /* hardcoded because used in message structure */
#define ORFS_ROOT_INO 1
#define ORFS_PROTOCOL_MAGIC 0x21071980

/* request type types */
typedef uint8_t       orfs_req_type_t;
typedef uint8_t       orfs_req_subtype_t;

/* other protocol specific types */
typedef uint64_t      orfs_tag_t;
typedef uint8_t       orfs_result_t;
typedef uint8_t       orfs_name_len_t;
typedef uint16_t      orfs_path_len_t;
typedef uint32_t      orfs_data_len_t;
typedef uint64_t      orfs_magic_t;
typedef uint32_t      orfs_proto_t;
typedef uint32_t      orfs_cap_t;
typedef uint64_t      orfs_file_id_t;

/* file system specific types */
typedef uint64_t      orfs_ino_t;
typedef uint8_t       orfs_sb_t;
typedef uint16_t      orfs_dev_t;
typedef uint16_t      orfs_mode_t;
typedef uint32_t      orfs_uid_t;
typedef uint32_t      orfs_gid_t;
typedef uint16_t      orfs_nlink_t;
typedef uint32_t      orfs_time_t;
typedef uint64_t      orfs_off_t; /* offset in file */
typedef uint64_t      orfs_doff_t; /* offset in directory entry list */
typedef uint32_t      orfs_valid_attr_t;
typedef uint8_t       orfs_dirent_type_t;
typedef uint8_t       orfs_bool_t;
typedef uint64_t      orfs_nfile_t;
typedef uint64_t      orfs_nblock_t;
typedef uint32_t      orfs_blksize_t;


/******************************
 * Inode and File Identifiers *
 ******************************/

struct orfs_file_id {
  orfs_file_id_t id;
};

struct orfs_inode_id {
  orfs_ino_t          ino; 
  orfs_sb_t           sb;
};


/**************
 * Attributes *
 **************/

struct orfs_getattr_data {
  struct orfs_inode_id id;
  orfs_dev_t           rdev;
  orfs_mode_t          mode;
  orfs_uid_t           uid;
  orfs_gid_t           gid;
  orfs_nlink_t         nlink;
  orfs_time_t          atime;
  orfs_time_t          mtime;
  orfs_time_t          ctime;
  orfs_off_t           size;
  orfs_off_t           blocks;
};


struct orfs_setattr_data {
  orfs_valid_attr_t     valid;
  orfs_mode_t           mode;
  orfs_uid_t            uid;
  orfs_gid_t            gid;
  orfs_off_t            size;
  orfs_time_t           atime;
  orfs_time_t           mtime;
  /*  orfs_time_t           ctime; maintained by the server */
  /*  uint32_t              attr_flags; useless ? only in intermezzo */
};


struct orfs_dirent {
  struct orfs_getattr_data attr;
  orfs_doff_t          off;
  orfs_dirent_type_t   type;
  orfs_name_len_t      name_len;
  char                 name[0];
};


struct orfs_fsattr_data {
  orfs_nblock_t blocks;
  orfs_nblock_t bfree;
  orfs_nblock_t bavail;
  orfs_nfile_t files;
  orfs_nfile_t ffree;
};


struct orfs_init_data {
  orfs_magic_t      magic;
  orfs_proto_t      protocol;
  orfs_blksize_t    blksize;
  orfs_path_len_t   symlen;
  orfs_cap_t        capabilities;
};


#endif /* __ORFS_TYPES__ */
