/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2004 Brice Goglin <Brice.Goglin@ens-lyon.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __RAMFS_MESSAGES_H__
#define __RAMFS_MESSAGES_H__


#include <stdio.h>


#define LOG_FILE stderr


#define INIT_LOGS() /* nothing */


/**********
 * Errors *
 **********/

#define ERROR(args...) fprintf(LOG_FILE, "ORFS RAM SERVER: " args)


/*********
 * Debug *
 *********/

#define LOG(level, args...)                  \
if ( level < DEBUG_LEVEL ) {                 \
int debug_int;                               \
for(debug_int=1;debug_int<level;debug_int++) \
fprintf(LOG_FILE, "..");                     \
fprintf(LOG_FILE, args);                     \
}


/*************
 * Asserting *
 *************/

#if ( DEBUG_LEVEL < 0 )
# define DEBUG_ASSERT(x) do { /* nothing */ } while ( 0 )
#else /* DEBUG_LEVEL >= 0 */
# include <assert.h>
# define DEBUG_ASSERT(x) assert(x)
#endif /* DEBUG_LEVEL >= 0 */


/************
 * Debuging *
 ************/

#if ( DEBUG_LEVEL < 0 )
# define DEBUG_INSTR(x) /* nothing */
#else /* DEBUG_LEVEL >= 0 */
# define DEBUG_INSTR(x) x
#endif /* DEBUG_LEVEL >= 0 */


#endif /* __RAMFS_MESSAGES_H__ */
