/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2004 Brice Goglin <Brice.Goglin@ens-lyon.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef __RAMFS_NETWORK_H__
#define __RAMFS_NETWORK_H__


#if CONFIG_MX
#include "myriexpress.h"
#endif


union orfs_connection {

#if CONFIG_TCP
  /* a  TCP connection is a socket */
  struct orfs_tcp_connection {
    int socket;
  } tcp;
#endif

#if CONFIG_GM
  /* a GM connection is a port and a node */
  struct orfs_gm_connection {
    int node;
    int port;
  } gm;
#endif

#if CONFIG_MX
  /* a MX connection is a mx_endpoint_addr_t */
  struct orfs_mx_connection {
    mx_endpoint_addr_t addr;
    uint32_t match_id;
  } mx;
#endif

};


extern int (*network_init)();
extern void (*network_exit)();
extern void (*network_process)(int *);
extern void* (*network_malloc)(size_t);
extern void (*network_free)(void*);


#if CONFIG_TCP

extern int TCP_init();
extern void TCP_exit();
extern void TCP_process(int *);

#define TCP_SETUP()				\
do {						\
	network_init = TCP_init;		\
	network_exit = TCP_exit;		\
	network_process = TCP_process;		\
	network_malloc = malloc;		\
	network_free = free;			\
} while ( 0 )

extern unsigned long tcp_server_port;

#endif /* CONFIG_TCP */


#if CONFIG_GM

extern int GM_init();
extern void GM_exit();
extern void GM_process(int *);
extern void *GM_malloc(size_t);
extern void GM_free(void*);

#define GM_SETUP();				\
do {						\
	network_init = GM_init;			\
	network_exit = GM_exit;			\
	network_process = GM_process;		\
	network_malloc = GM_malloc;		\
	network_free = GM_free;			\
} while ( 0 )

extern unsigned long gm_server_port;
extern unsigned long gm_server_board;

#endif /* CONFIG_GM */


#if CONFIG_MX

extern int MX_init();
extern void MX_exit();
extern void MX_process(int *);

#define MX_SETUP();				\
do {						\
	network_init = MX_init;			\
	network_exit = MX_exit;			\
	network_process = MX_process;		\
	network_malloc = malloc;		\
	network_free = free;			\
} while ( 0 )

extern unsigned long mx_server_ep;
extern unsigned long mx_server_board;

#endif /* CONFIG_MX */


#endif /* __RAMFS_NETWORK_H__ */
