/*
 * ORFA - A user-level remote file system access protocol
 * Copyright (C) 2004 Brice Goglin <Brice.Goglin@ens-lyon.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <stdlib.h>

#include "ramfs_server.h"
#include "ramfs_network.h"
#include "ramfs_list.h"
#include "ramfs_queue.h"
#include "ramfs_messages.h"


/**********
 * Queues *
 **********/

DECLARE_REQUEST_QUEUE(free_request_queue);
DECLARE_REQUEST_QUEUE(ready_request_queue);
DECLARE_REQUEST_QUEUE(done_request_queue);


void queue_init()
{
  INIT_REQUEST_QUEUE(free_request_queue);
  INIT_REQUEST_QUEUE(ready_request_queue);
  INIT_REQUEST_QUEUE(done_request_queue);
}


/*****************
 * Free Requests *
 *****************/

static void alloc_new_free_request()
{
  int i;
  struct server_request *new = (struct server_request*)
    network_malloc(QUEUE_ALLOCATING_NUMBER*sizeof(*new));

  if ( !new ) {
    ERROR("Failed to allocate new requests\n");
    return;
  }

  for(i=0; i<QUEUE_ALLOCATING_NUMBER; i++) {
#if DEBUG_LEVEL >= 0
    new[i].queue.prev = NULL;
    new[i].queue.next = NULL;
#endif
    new[i].count = 0;
    QUEUE_REQUEST(&free_request_queue, &new[i]);
  }

  LOG(4, "Allocated %d new request\n", QUEUE_ALLOCATING_NUMBER);
}


struct server_request *get_free_request()
{
  struct server_request *req;

  if ( QUEUE_EMPTY(&free_request_queue) )
    alloc_new_free_request();

  req = DEQUEUE_REQUEST(&free_request_queue);
  if ( !req )
    return NULL;

  /* we do not clear the whole request,
   * because it's too big, and not really usefull.
   */
  req->in_vect = NULL;
  req->out_vect = NULL;

  return req;
}


void put_free_request(struct server_request *req)
{
  assert(!req->count); 
  QUEUE_REQUEST(&free_request_queue, req);
}
